@@ -15,7 +15,7 @@
 #define lex_mode__Undefined 1
 #define lex_mode__Comment 2
 #define lex_mode__ShCommand 3
-#define lex_mode__ShCommandBrack 4
+#define lex_mode__ShCommandFakeBrack 4
 #define lex_mode__Backtick 5
 #define lex_mode__DBracket 6
 #define lex_mode__SQ_Raw 7
@@ -28,12 +28,13 @@
 #define lex_mode__VSub_2 14
 #define lex_mode__VSub_ArgUnquoted 15
 #define lex_mode__VSub_ArgDQ 16
-#define lex_mode__BashRegex 17
-#define lex_mode__BashRegexChars 18
-#define lex_mode__FuncParens 19
-#define lex_mode__PrintfOuter 20
-#define lex_mode__PrintfPercent 21
-#define lex_mode__Expr 22
+#define lex_mode__VSub_Zsh 17
+#define lex_mode__BashRegex 18
+#define lex_mode__BashRegexFakeInner 19
+#define lex_mode__FuncParens 20
+#define lex_mode__PrintfOuter 21
+#define lex_mode__PrintfPercent 22
+#define lex_mode__Expr 23
 
 #define word_mode__ShCommand 1
 #define word_mode__ShCommandBrack 2
