@@ -3,11 +3,12 @@ alloc.py - strategies for managing SourceLine and Token
 
 """
 
-from _devbuild.gen.syntax_asdl import source_t, Token, SourceLine
+from _devbuild.gen.syntax_asdl import source_t, Token, SourceLine, loc
 from asdl import runtime
+from core import error
 from mycpp.mylib import log
 
-from typing import List, Any
+from typing import List, Dict, Any
 
 _ = log
 
@@ -96,6 +97,9 @@ class Arena(object):
         self.tokens = []  # type: List[Token]
         self.num_tokens = 0
 
+        # Only used in tools
+        self.span_id_lookup = {}  # type: Dict[Token, int]
+
         # All lines that haven't been discarded.  For LST formatting.
         self.lines_list = []  # type: List[SourceLine]
 
@@ -239,14 +243,21 @@ class Arena(object):
         assert found_right, "Couldn't find right token"
         return ''.join(pieces)
 
-    def NewToken(self, id_, col, length, src_line, val):
-        # type: (int, int, int, SourceLine, str) -> Token
-        span_id = self.num_tokens  # spids are just array indices
-        self.num_tokens += 1
+    def NewToken(self, id_, col, length, src_line):
+        # type: (int, int, int, SourceLine) -> Token
 
-        tok = Token(id_, col, length, span_id, src_line, val)
+        if length >= 65536:
+            raise error.Parse(
+                '',  # ignored message
+                loc.TokenTooLong(src_line, id_, length, col))
+
+        tok = Token(id_, length, col, src_line, None)
         if self.save_tokens:
+            span_id = self.num_tokens
+            self.num_tokens += 1
+
             self.tokens.append(tok)
+            self.span_id_lookup[tok] = span_id
         return tok
 
     def UnreadOne(self):
@@ -254,7 +265,7 @@ class Arena(object):
         """Reuse the last span ID."""
         if self.save_tokens:
             self.tokens.pop()
-        self.num_tokens -= 1
+            self.num_tokens -= 1
 
     def GetToken(self, span_id):
         # type: (int) -> Token
@@ -263,6 +274,14 @@ class Arena(object):
           'Span ID out of range: %d is greater than %d' % (span_id, len(self.tokens))
         return self.tokens[span_id]
 
+    def GetSpanId(self, tok):
+        # type: (Token) -> int
+        """Given a Token, returns its a sequence number"""
+        #return tok.span_id
+        #return -1
+        assert tok in self.span_id_lookup
+        return self.span_id_lookup[tok]
+
     def LastSpanId(self):
         # type: () -> int
         """Return one past the last span ID."""
