@@ -8,7 +8,7 @@ from _devbuild.gen.value_asdl import (value, value_e)
 from core import completion
 from core import error
 from core import state
-from core import ui
+from display import ui
 from core import vm
 from mycpp import mylib
 from mycpp.mylib import log, print_stderr
@@ -22,7 +22,7 @@ from typing import Dict, List, Iterator, cast, TYPE_CHECKING
 if TYPE_CHECKING:
     from _devbuild.gen.runtime_asdl import cmd_value
     from core.completion import Lookup, OptionState, Api, UserSpec
-    from core.ui import ErrorFormatter
+    from display import ui
     from frontend.args import _Attributes
     from frontend.parse_lib import ParseContext
     from osh.cmd_eval import CommandEvaluator
@@ -55,12 +55,12 @@ class _DynamicProcDictAction(completion.CompletionAction):
     """
 
     def __init__(self, d):
-        # type: (Dict[str, value.Proc]) -> None
+        # type: (state.Procs) -> None
         self.d = d
 
     def Matches(self, comp):
         # type: (Api) -> Iterator[str]
-        for name in sorted(self.d):
+        for name in self.d.GetNames():
             if name.startswith(comp.to_complete):
                 yield name
 
@@ -139,7 +139,7 @@ class SpecBuilder(object):
         # obviously it's better to check here.
         if arg.F is not None:
             func_name = arg.F
-            func = cmd_ev.procs.get(func_name)
+            func = cmd_ev.procs.Get(func_name)
             if func is None:
                 raise error.Usage('function %r not found' % func_name,
                                   loc.Missing)
@@ -405,7 +405,7 @@ class CompOpt(vm._Builtin):
     """Adjust options inside user-defined completion functions."""
 
     def __init__(self, comp_state, errfmt):
-        # type: (OptionState, ErrorFormatter) -> None
+        # type: (OptionState, ui.ErrorFormatter) -> None
         self.comp_state = comp_state
         self.errfmt = errfmt
 
