@@ -14,7 +14,6 @@ from _devbuild.gen.syntax_asdl import (
     loc,
     loc_t,
 )
-from _devbuild.gen.value_asdl import value
 from builtin import hay_ysh
 from core import dev
 from core import error
@@ -23,15 +22,15 @@ from core.error import e_die, e_die_status
 from core import pyos
 from core import pyutil
 from core import state
-from core import ui
+from display import ui
 from core import vm
 from frontend import consts
 from frontend import lexer
-from mycpp.mylib import log
+from mycpp.mylib import log, print_stderr
 
 import posix_ as posix
 
-from typing import cast, Dict, List, Optional, TYPE_CHECKING
+from typing import cast, Dict, List, Tuple, Optional, TYPE_CHECKING
 if TYPE_CHECKING:
     from _devbuild.gen.runtime_asdl import (cmd_value, CommandStatus,
                                             StatusArray)
@@ -89,8 +88,8 @@ class _ProcessSubFrame(object):
         status_array.locs = locs
 
 
-# Big flgas for RunSimpleCommand
-DO_FORK = 1 << 1
+# Big flags for RunSimpleCommand
+IS_LAST_CMD = 1 << 1
 NO_CALL_PROCS = 1 << 2  # command ls suppresses function lookup
 USE_DEFAULT_PATH = 1 << 3  # for command -p ls changes the path
 
@@ -110,7 +109,7 @@ class ShellExecutor(vm._Executor):
             mem,  # type: state.Mem
             exec_opts,  # type: optview.Exec
             mutable_opts,  # type: state.MutableOpts
-            procs,  # type: Dict[str, value.Proc]
+            procs,  # type: state.Procs
             hay_state,  # type: hay_ysh.HayState
             builtins,  # type: Dict[int, _Builtin]
             search_path,  # type: state.SearchPath
@@ -158,8 +157,8 @@ class ShellExecutor(vm._Executor):
         # type: () -> None
         assert self.cmd_ev is not None
 
-    def _MakeProcess(self, node, inherit_errexit=True):
-        # type: (command_t, bool) -> process.Process
+    def _MakeProcess(self, node, inherit_errexit, inherit_errtrace):
+        # type: (command_t, bool, bool) -> process.Process
         """Assume we will run the node in another process.
 
         Return a process.
@@ -185,11 +184,9 @@ class ShellExecutor(vm._Executor):
         #   interleaved.
         # - We could turn the `exit` builtin into a error.FatalRuntime exception
         #   and get this check for "free".
-        thunk = process.SubProgramThunk(self.cmd_ev,
-                                        node,
-                                        self.trap_state,
-                                        self.multi_trace,
-                                        inherit_errexit=inherit_errexit)
+        thunk = process.SubProgramThunk(self.cmd_ev, node, self.trap_state,
+                                        self.multi_trace, inherit_errexit,
+                                        inherit_errtrace)
         p = process.Process(thunk, self.job_control, self.job_list,
                             self.tracer)
         return p
@@ -282,7 +279,7 @@ class ShellExecutor(vm._Executor):
             # Pitfall: What happens if there are two of the same name?  I guess
             # that's why you have = and 'type' inspect them
 
-            proc_node = self.procs.get(arg0)
+            proc_node = self.procs.Get(arg0)
             if proc_node is not None:
                 if self.exec_opts.strict_errexit():
                     disabled_tok = self.mutable_opts.ErrExitDisabledToken()
@@ -329,10 +326,10 @@ class ShellExecutor(vm._Executor):
 
         environ = self.mem.GetExported()  # Include temporary variables
 
-        if cmd_val.typed_args:
+        if cmd_val.proc_args:
             e_die(
                 '%r appears to be external. External commands don\'t accept typed args (OILS-ERR-200)'
-                % arg0, cmd_val.typed_args.left)
+                % arg0, cmd_val.proc_args.typed_args.left)
 
         # Resolve argv[0] BEFORE forking.
         if run_flags & USE_DEFAULT_PATH:
@@ -343,8 +340,13 @@ class ShellExecutor(vm._Executor):
             self.errfmt.Print_('%r not found (OILS-ERR-100)' % arg0, arg0_loc)
             return 127
 
+        if self.trap_state.ThisProcessHasTraps():
+            do_fork = True
+        else:
+            do_fork = not cmd_val.is_last_cmd
+
         # Normal case: ls /
-        if run_flags & DO_FORK:
+        if do_fork:
             thunk = process.ExternalThunk(self.ext_prog, argv0_path, cmd_val,
                                           environ)
             p = process.Process(thunk, self.job_control, self.job_list,
@@ -396,7 +398,7 @@ class ShellExecutor(vm._Executor):
             pi = process.Pipeline(self.exec_opts.sigpipe_status_ok(),
                                   self.job_control, self.job_list, self.tracer)
             for child in node.children:
-                p = self._MakeProcess(child)
+                p = self._MakeProcess(child, True, self.exec_opts.errtrace())
                 p.Init_ParentPipeline(pi)
                 pi.Add(p)
 
@@ -405,14 +407,14 @@ class ShellExecutor(vm._Executor):
             last_pid = pi.LastPid()
             self.mem.last_bg_pid = last_pid  # for $!
 
-            self.job_list.AddJob(pi)  # show in 'jobs' list
+            job_id = self.job_list.AddJob(pi)  # show in 'jobs' list
 
         else:
             # Problem: to get the 'set -b' behavior of immediate notifications, we
             # have to register SIGCHLD.  But then that introduces race conditions.
             # If we haven't called Register yet, then we won't know who to notify.
 
-            p = self._MakeProcess(node)
+            p = self._MakeProcess(node, True, self.exec_opts.errtrace())
             if self.job_control.Enabled():
                 p.AddStateChange(
                     process.SetPgid(process.OWN_LEADER, self.tracer))
@@ -420,7 +422,13 @@ class ShellExecutor(vm._Executor):
             p.SetBackground()
             pid = p.StartProcess(trace.Fork)
             self.mem.last_bg_pid = pid  # for $!
-            self.job_list.AddJob(p)  # show in 'jobs' list
+            job_id = self.job_list.AddJob(p)  # show in 'jobs' list
+
+        if self.exec_opts.interactive():
+            # Print it like %1 to show it's a job
+            print_stderr('[%%%d] PID %d Started' %
+                         (job_id, self.mem.last_bg_pid))
+
         return 0
 
     def RunPipeline(self, node, status_out):
@@ -440,7 +448,7 @@ class ShellExecutor(vm._Executor):
             # TODO: determine these locations at parse time?
             pipe_locs.append(loc.Command(child))
 
-            p = self._MakeProcess(child)
+            p = self._MakeProcess(child, True, self.exec_opts.errtrace())
             p.Init_ParentPipeline(pi)
             pi.Add(p)
 
@@ -459,43 +467,17 @@ class ShellExecutor(vm._Executor):
 
     def RunSubshell(self, node):
         # type: (command_t) -> int
-        p = self._MakeProcess(node)
+        p = self._MakeProcess(node, True, self.exec_opts.errtrace())
         if self.job_control.Enabled():
             p.AddStateChange(process.SetPgid(process.OWN_LEADER, self.tracer))
 
         return p.RunProcess(self.waiter, trace.ForkWait)
 
-    def RunCommandSub(self, cs_part):
-        # type: (CommandSub) -> str
+    def CaptureStdout(self, node):
+        # type: (command_t) -> Tuple[int, str]
 
-        if not self.exec_opts._allow_command_sub():
-            # _allow_command_sub is used in two places.  Only one of them turns off _allow_process_sub
-            if not self.exec_opts._allow_process_sub():
-                why = "status wouldn't be checked (strict_errexit)"
-            else:
-                why = 'eval_unsafe_arith is off'
-
-            e_die("Command subs not allowed here because %s" % why,
-                  loc.WordPart(cs_part))
-
-        node = cs_part.child
-
-        # Hack for weird $(<file) construct
-        if node.tag() == command_e.Simple:
-            simple = cast(command.Simple, node)
-            # Detect '< file'
-            if (len(simple.words) == 0 and len(simple.redirects) == 1 and
-                    simple.redirects[0].op.id == Id.Redir_Less):
-                # change it to __cat < file
-                # TODO: change to 'internal cat' (issue 1013)
-                tok = lexer.DummyToken(Id.Lit_Chars, '__cat')
-                cat_word = CompoundWord([tok])
-                # MUTATE the command.Simple node.  This will only be done the first
-                # time in the parent process.
-                simple.words.append(cat_word)
-
-        p = self._MakeProcess(node,
-                              inherit_errexit=self.exec_opts.inherit_errexit())
+        p = self._MakeProcess(node, self.exec_opts.inherit_errexit(),
+                              self.exec_opts.errtrace())
         # Shell quirk: Command subs remain part of the shell's process group, so we
         # don't use p.AddStateChange(process.SetPgid(...))
 
@@ -517,13 +499,55 @@ class ShellExecutor(vm._Executor):
                     # Like the top level IOError handler
                     e_die_status(
                         2,
-                        'osh I/O error (read): %s' % posix.strerror(err_num))
+                        'Oils I/O error (read): %s' % posix.strerror(err_num))
 
             elif n == 0:  # EOF
                 break
         posix.close(r)
 
         status = p.Wait(self.waiter)
+        stdout_str = ''.join(chunks).rstrip('\n')
+
+        return status, stdout_str
+
+    def RunCommandSub(self, cs_part):
+        # type: (CommandSub) -> str
+
+        if not self.exec_opts._allow_command_sub():
+            # _allow_command_sub is used in two places.  Only one of them turns
+            # off _allow_process_sub
+            if not self.exec_opts._allow_process_sub():
+                why = "status wouldn't be checked (strict_errexit)"
+            else:
+                why = 'eval_unsafe_arith is off'
+
+            e_die("Command subs not allowed here because %s" % why,
+                  loc.WordPart(cs_part))
+
+        node = cs_part.child
+
+        # Hack for weird $(<file) construct
+        if node.tag() == command_e.Redirect:
+            redir_node = cast(command.Redirect, node)
+            # Detect '< file'
+            if (len(redir_node.redirects) == 1 and
+                    redir_node.redirects[0].op.id == Id.Redir_Less and
+                    redir_node.child.tag() == command_e.NoOp):
+
+                # Change it to __cat < file.
+                # TODO: could be 'internal cat' (issue #1013)
+                tok = lexer.DummyToken(Id.Lit_Chars, '__cat')
+                cat_word = CompoundWord([tok])
+
+                # Blame < because __cat has no location
+                blame_tok = redir_node.redirects[0].op
+                simple = command.Simple(blame_tok, [], [cat_word], None, None,
+                                        False)
+
+                # MUTATE redir node so it's like $(<file _cat)
+                redir_node.child = simple
+
+        status, stdout_str = self.CaptureStdout(node)
 
         # OSH has the concept of aborting in the middle of a WORD.  We're not
         # waiting until the command is over!
@@ -547,7 +571,7 @@ class ShellExecutor(vm._Executor):
         # Runtime errors test case: # $("echo foo > $@")
         # Why rstrip()?
         # https://unix.stackexchange.com/questions/17747/why-does-shell-command-substitution-gobble-up-a-trailing-newline-char
-        return ''.join(chunks).rstrip('\n')
+        return stdout_str
 
     def RunProcessSub(self, cs_part):
         # type: (CommandSub) -> str
@@ -600,7 +624,7 @@ class ShellExecutor(vm._Executor):
                 "Process subs not allowed here because status wouldn't be checked (strict_errexit)",
                 cs_loc)
 
-        p = self._MakeProcess(cs_part.child)
+        p = self._MakeProcess(cs_part.child, True, self.exec_opts.errtrace())
 
         r, w = posix.pipe()
         #log('pipe = %d, %d', r, w)
