@@ -45,13 +45,14 @@ from _devbuild.gen.runtime_asdl import (
     Piece,
 )
 from _devbuild.gen.value_asdl import (value, value_e, value_t, y_lvalue,
-                                      y_lvalue_e, y_lvalue_t, IntBox, LeftName)
+                                      y_lvalue_e, y_lvalue_t, IntBox, LeftName,
+                                      Obj)
 from core import error
 from core.error import e_die, e_die_status
 from core import num
 from core import pyutil
 from core import state
-from core import ui
+from display import ui
 from core import vm
 from data_lang import j8
 from frontend import lexer
@@ -230,15 +231,37 @@ class ExprEvaluator(object):
                         index = val_ops.ToInt(lval.index,
                                               'List index should be Int',
                                               loc.Missing)
-                        lhs_val_ = obj.items[index]
+                        try:
+                            lhs_val_ = obj.items[index]
+                        except IndexError:
+                            raise error.Expr(
+                                'List index out of range: %d' % index,
+                                loc.Missing)
 
                     elif case(value_e.Dict):
                         obj = cast(value.Dict, UP_obj)
                         index = -1  # silence C++ warning
                         key = val_ops.ToStr(lval.index,
-                                            'Dict index should be Str',
+                                            'Dict key should be Str',
                                             loc.Missing)
-                        lhs_val_ = obj.d[key]
+                        try:
+                            lhs_val_ = obj.d[key]
+                        except KeyError:
+                            raise error.Expr('Dict key not found: %r' % key,
+                                             loc.Missing)
+
+                    elif case(value_e.Obj):
+                        obj = cast(Obj, UP_obj)
+                        index = -1  # silence C++ warning
+                        key = val_ops.ToStr(lval.index,
+                                            'Obj attribute should be Str',
+                                            loc.Missing)
+                        try:
+                            lhs_val_ = obj.d[key]
+                        except KeyError:
+                            raise error.Expr(
+                                'Obj attribute not found: %r' % key,
+                                loc.Missing)
 
                     else:
                         raise error.TypeErr(
@@ -261,6 +284,13 @@ class ExprEvaluator(object):
                         obj = cast(value.Dict, UP_obj)
                         obj.d[key] = new_val_
 
+                    elif case(value_e.Obj):
+                        obj = cast(Obj, UP_obj)
+                        obj.d[key] = new_val_
+
+                    else:
+                        raise AssertionError()
+
             else:
                 raise AssertionError()
 
@@ -535,13 +565,13 @@ class ExprEvaluator(object):
                     # Disallow this to remove confusion between modulus and remainder
                     raise error.Expr("Divisor can't be negative", op)
 
-                return value.Int(num.IntRemainder(i1, i2))
+                return value.Int(mops.Rem(i1, i2))
 
             # a // b   setvar a //= b
             elif case(Id.Expr_DSlash, Id.Expr_DSlashEqual):
                 if mops.Equal(i2, mops.ZERO):
                     raise error.Expr('Divide by zero', op)
-                return value.Int(num.IntDivide(i1, i2))
+                return value.Int(mops.Div(i1, i2))
 
             # a ** b   setvar a **= b (ysh only)
             elif case(Id.Arith_DStar, Id.Expr_DStarEqual):
@@ -561,9 +591,14 @@ class ExprEvaluator(object):
                 return value.Int(mops.BitXor(i1, i2))
 
             elif case(Id.Arith_DGreat, Id.Arith_DGreatEqual):  # >>
+                if mops.Greater(mops.ZERO, i2):  # i2 < 0
+                    raise error.Expr("Can't right shift by negative number",
+                                     op)
                 return value.Int(mops.RShift(i1, i2))
 
             elif case(Id.Arith_DLess, Id.Arith_DLessEqual):  # <<
+                if mops.Greater(mops.ZERO, i2):  # i2 < 0
+                    raise error.Expr("Can't left shift by negative number", op)
                 return value.Int(mops.LShift(i1, i2))
 
             else:
@@ -676,15 +711,9 @@ class ExprEvaluator(object):
                 result = self._CompareNumeric(left, right, op)
 
             elif op.id == Id.Expr_TEqual:
-                if left.tag() != right.tag():
-                    result = False
-                else:
-                    result = val_ops.ExactlyEqual(left, right, op)
+                result = val_ops.ExactlyEqual(left, right, op)
             elif op.id == Id.Expr_NotDEqual:
-                if left.tag() != right.tag():
-                    result = True
-                else:
-                    result = not val_ops.ExactlyEqual(left, right, op)
+                result = not val_ops.ExactlyEqual(left, right, op)
 
             elif op.id == Id.Expr_In:
                 result = val_ops.Contains(left, right)
@@ -895,7 +924,8 @@ class ExprEvaluator(object):
                             return obj.items[i]
                         except IndexError:
                             # TODO: expr.Subscript has no error location
-                            raise error.Expr('index out of range', loc.Missing)
+                            raise error.Expr('List index out of range: %d' % i,
+                                             loc.Missing)
 
                     else:
                         raise error.TypeErr(
@@ -913,61 +943,153 @@ class ExprEvaluator(object):
                     return obj.d[index.s]
                 except KeyError:
                     # TODO: expr.Subscript has no error location
-                    raise error.Expr('Dict entry %r not found' % index.s,
+                    raise error.Expr('Dict entry not found: %r' % index.s,
                                      loc.Missing)
 
         raise error.TypeErr(obj, 'Subscript expected Str, List, or Dict',
                             loc.Missing)
 
-    def _EvalDot(self, node, obj):
-        # type: (Attribute, value_t) -> value_t
-        """ obj.attr on RHS or LHS
+    def _ChainedLookup(self, obj, current, attr_name):
+        # type: (Obj, Obj, str) -> Optional[value_t]
+        """Prototype chain lookup.
 
-        setvar x = obj.attr
-        setglobal g[obj.attr] = 42
+        Args:
+          obj: properties we might bind to
+          current: our location in the prototype chain
         """
-        UP_obj = obj
-        with tagswitch(obj) as case:
+        val = current.d.get(attr_name)
+        if val is not None:
+            # Special bound method logic for objects, but NOT modules
+            if val.tag() in (value_e.Func, value_e.BuiltinFunc):
+                return value.BoundFunc(obj, val)
+            else:
+                return val
+
+        if current.prototype is not None:
+            return self._ChainedLookup(obj, current.prototype, attr_name)
+
+        return None
+
+    def _EvalDot(self, node, val):
+        # type: (Attribute, value_t) -> value_t
+        """ foo.attr on RHS or LHS
+
+        setvar x = foo.attr
+        setglobal g[foo.attr] = 42
+        """
+        UP_val = val
+        with tagswitch(val) as case:
             if case(value_e.Dict):
-                obj = cast(value.Dict, UP_obj)
+                val = cast(value.Dict, UP_val)
                 attr_name = node.attr_name
-                try:
-                    result = obj.d[attr_name]
-                except KeyError:
-                    raise error.Expr('Dict entry %r not found' % attr_name,
-                                     node.op)
+
+                # Dict key / normal attribute lookup
+                result = val.d.get(attr_name)
+                if result is not None:
+                    return result
+
+                raise error.Expr('Dict entry %r not found' % attr_name,
+                                 node.op)
+
+            elif case(value_e.Obj):
+                obj = cast(Obj, UP_val)
+                attr_name = node.attr_name
+
+                # Dict key / normal attribute lookup
+                result = obj.d.get(attr_name)
+                if result is not None:
+                    return result
+
+                # Prototype lookup - with special logic for BoundMethod
+                if obj.prototype is not None:
+                    result = self._ChainedLookup(obj, obj.prototype, attr_name)
+                    if result is not None:
+                        return result
+
+                raise error.Expr('Attribute %r not found on Obj' % attr_name,
+                                 node.op)
 
             else:
-                raise error.TypeErr(obj, 'Dot operator expected Dict', node.op)
-
-        return result
-
-    def _EvalAttribute(self, node):
-        # type: (Attribute) -> value_t
-
-        o = self._EvalExpr(node.obj)
-        UP_o = o
-
-        with switch(node.op.id) as case:
-            # Right now => is a synonym for ->
-            # Later we may enforce that => is pure, and -> is for mutation and
-            # I/O.
-            if case(Id.Expr_RArrow, Id.Expr_RDArrow):
+                # Method lookup on builtin types.
+                # They don't have attributes or prototype chains -- we only
+                # have a flat dict.
+                type_methods = self.methods.get(val.tag())
                 name = node.attr_name
-                # Look up builtin methods
-                type_methods = self.methods.get(o.tag())
                 vm_callable = (type_methods.get(name)
                                if type_methods is not None else None)
                 if vm_callable:
                     func_val = value.BuiltinFunc(vm_callable)
-                    return value.BoundFunc(o, func_val)
+                    return value.BoundFunc(val, func_val)
 
-                # If the operator is ->, fail because we don't have any
-                # user-defined methods
-                if node.op.id == Id.Expr_RArrow:
-                    raise error.TypeErrVerbose(
-                        'Method %r does not exist on type %s' %
-                        (name, ui.ValType(o)), node.attr)
+                raise error.TypeErrVerbose(
+                    "Method %r not found on builtin type %s" %
+                    (name, ui.ValType(val)), node.attr)
+
+        raise AssertionError()
+
+    def _EvalRArrow(self, node, val):
+        # type: (Attribute, value_t) -> value_t
+        mut_name = 'M/' + node.attr_name
+
+        UP_val = val
+        with tagswitch(val) as case:
+            if case(value_e.Obj):
+                obj = cast(Obj, UP_val)
+
+                if obj.prototype is not None:
+                    result = self._ChainedLookup(obj, obj.prototype, mut_name)
+                    if result is not None:
+                        return result
+
+                # TODO: we could have different errors for:
+                # - no prototype
+                # - found in the properties, not in the prototype chain (not
+                #   sure if this error is common.)
+                raise error.Expr(
+                    "Mutating method %r not found on Obj prototype chain" % mut_name,
+                    node.attr)
+            else:
+                # Look up methods on builtin types
+                # TODO: These should also be called M/append, M/erase, etc.
+
+                type_methods = self.methods.get(val.tag())
+                vm_callable = (type_methods.get(mut_name)
+                               if type_methods is not None else None)
+                if vm_callable:
+                    func_val = value.BuiltinFunc(vm_callable)
+                    return value.BoundFunc(val, func_val)
+
+                raise error.TypeErrVerbose(
+                    "Mutating method %r not found on builtin type %s" %
+                    (mut_name, ui.ValType(val)), node.attr)
+        raise AssertionError()
+
+    def _EvalAttribute(self, node):
+        # type: (Attribute) -> value_t
+
+        val = self._EvalExpr(node.obj)
+        with switch(node.op.id) as case:
+            if case(Id.Expr_Dot):  # d.key is like d['key']
+                return self._EvalDot(node, val)
+
+            elif case(Id.Expr_RArrow):  # e.g. mylist->append(42)
+                return self._EvalRArrow(node, val)
+
+            elif case(Id.Expr_RDArrow):  # chaining s => split()
+                name = node.attr_name
+
+                # Look up builtin methods, e.g.
+                #   s => strip() is like s.strip()
+                # Note:
+                #   m => group(1) is worse than m.group(1)
+                #   This is not a transformation, but more like an attribute
+
+                type_methods = self.methods.get(val.tag())
+                vm_callable = (type_methods.get(name)
+                               if type_methods is not None else None)
+                if vm_callable:
+                    func_val = value.BuiltinFunc(vm_callable)
+                    return value.BoundFunc(val, func_val)
 
                 # Operator is =>, so try function chaining.
 
@@ -978,21 +1100,19 @@ class ExprEvaluator(object):
                 #     f() => str() => upper()
 
                 # Could improve error message: may give "Undefined variable"
-                val = self._LookupVar(name, node.attr)
+                val2 = self._LookupVar(name, node.attr)
 
-                with tagswitch(val) as case2:
+                with tagswitch(val2) as case2:
                     if case2(value_e.Func, value_e.BuiltinFunc):
-                        return value.BoundFunc(o, val)
+                        return value.BoundFunc(val, val2)
                     else:
                         raise error.TypeErr(
-                            val, 'Fat arrow => expects method or function',
+                            val2, 'Fat arrow => expects method or function',
                             node.attr)
 
-            elif case(Id.Expr_Dot):  # d.key is like d['key']
-                return self._EvalDot(node, o)
-
             else:
                 raise AssertionError(node.op)
+        raise AssertionError()
 
     def _EvalExpr(self, node):
         # type: (expr_t) -> value_t
