@@ -162,9 +162,10 @@ ID_TO_KIND = {
   Id.Ignored_LineCont: Kind.Ignored,
   Id.Ignored_Space: Kind.Ignored,
   Id.Ignored_Comment: Kind.Ignored,
-  Id.Ignored_HereTabs: Kind.Ignored,
+  Id.Ignored_Newline: Kind.Ignored,
   Id.WS_Space: Kind.WS,
   Id.Lit_Chars: Kind.Lit,
+  Id.Lit_CharsWithoutPrefix: Kind.Lit,
   Id.Lit_VarLike: Kind.Lit,
   Id.Lit_ArrayLhsOpen: Kind.Lit,
   Id.Lit_ArrayLhsClose: Kind.Lit,
@@ -173,7 +174,6 @@ ID_TO_KIND = {
   Id.Lit_AtLBraceDot: Kind.Lit,
   Id.Lit_Other: Kind.Lit,
   Id.Lit_EscapedChar: Kind.Lit,
-  Id.Lit_RegexMeta: Kind.Lit,
   Id.Lit_LBracket: Kind.Lit,
   Id.Lit_RBracket: Kind.Lit,
   Id.Lit_Star: Kind.Lit,
@@ -212,6 +212,8 @@ ID_TO_KIND = {
   Id.Op_DPipe: Kind.Op,
   Id.Op_Semi: Kind.Op,
   Id.Op_DSemi: Kind.Op,
+  Id.Op_SemiAmp: Kind.Op,
+  Id.Op_DSemiAmp: Kind.Op,
   Id.Op_LParen: Kind.Op,
   Id.Op_RParen: Kind.Op,
   Id.Op_DLeftParen: Kind.Op,
@@ -265,10 +267,6 @@ ID_TO_KIND = {
   Id.Expr_Func: Kind.Expr,
   Id.Expr_Capture: Kind.Expr,
   Id.Expr_As: Kind.Expr,
-  Id.Expr_While: Kind.Expr,
-  Id.Expr_Break: Kind.Expr,
-  Id.Expr_Continue: Kind.Expr,
-  Id.Expr_Return: Kind.Expr,
   Id.Char_OneChar: Kind.Char,
   Id.Char_Stop: Kind.Char,
   Id.Char_Hex: Kind.Char,
@@ -280,11 +278,12 @@ ID_TO_KIND = {
   Id.Char_Unicode8: Kind.Char,
   Id.Char_UBraced: Kind.Char,
   Id.Char_Pound: Kind.Char,
-  Id.Char_Literals: Kind.Char,
   Id.Char_AsciiControl: Kind.Char,
-  Id.Re_Start: Kind.Re,
-  Id.Re_End: Kind.Re,
-  Id.Re_Dot: Kind.Re,
+  Id.BashRegex_LParen: Kind.BashRegex,
+  Id.BashRegex_AllowedInParens: Kind.BashRegex,
+  Id.Eggex_Start: Kind.Eggex,
+  Id.Eggex_End: Kind.Eggex,
+  Id.Eggex_Dot: Kind.Eggex,
   Id.Redir_Less: Kind.Redir,
   Id.Redir_Great: Kind.Redir,
   Id.Redir_DLess: Kind.Redir,
@@ -298,12 +297,14 @@ ID_TO_KIND = {
   Id.Redir_AndGreat: Kind.Redir,
   Id.Redir_AndDGreat: Kind.Redir,
   Id.Left_DoubleQuote: Kind.Left,
+  Id.Left_JDoubleQuote: Kind.Left,
   Id.Left_SingleQuote: Kind.Left,
   Id.Left_DollarSingleQuote: Kind.Left,
   Id.Left_RSingleQuote: Kind.Left,
   Id.Left_USingleQuote: Kind.Left,
   Id.Left_BSingleQuote: Kind.Left,
   Id.Left_TDoubleQuote: Kind.Left,
+  Id.Left_DollarTDoubleQuote: Kind.Left,
   Id.Left_TSingleQuote: Kind.Left,
   Id.Left_RTSingleQuote: Kind.Left,
   Id.Left_UTSingleQuote: Kind.Left,
@@ -311,6 +312,7 @@ ID_TO_KIND = {
   Id.Left_Backtick: Kind.Left,
   Id.Left_DollarParen: Kind.Left,
   Id.Left_DollarBrace: Kind.Left,
+  Id.Left_DollarBraceZsh: Kind.Left,
   Id.Left_DollarDParen: Kind.Left,
   Id.Left_DollarBracket: Kind.Left,
   Id.Left_DollarDoubleQuote: Kind.Left,
@@ -335,6 +337,7 @@ ID_TO_KIND = {
   Id.Right_CasePat: Kind.Right,
   Id.Right_ShArrayLiteral: Kind.Right,
   Id.Right_ExtGlob: Kind.Right,
+  Id.Right_BashRegexGroup: Kind.Right,
   Id.Right_BlockLiteral: Kind.Right,
   Id.ExtGlob_Comma: Kind.ExtGlob,
   Id.ExtGlob_At: Kind.ExtGlob,
@@ -374,8 +377,8 @@ ID_TO_KIND = {
   Id.VOp1_DCaret: Kind.VOp1,
   Id.VOp1_Comma: Kind.VOp1,
   Id.VOp1_DComma: Kind.VOp1,
-  Id.VOpOil_Pipe: Kind.VOpOil,
-  Id.VOpOil_Space: Kind.VOpOil,
+  Id.VOpYsh_Pipe: Kind.VOpYsh,
+  Id.VOpYsh_Space: Kind.VOpYsh,
   Id.VOp2_Slash: Kind.VOp2,
   Id.VOp2_Colon: Kind.VOp2,
   Id.VOp2_LBracket: Kind.VOp2,
@@ -411,6 +414,7 @@ ID_TO_KIND = {
   Id.KW_SetGlobal: Kind.KW,
   Id.KW_Call: Kind.KW,
   Id.KW_Proc: Kind.KW,
+  Id.KW_Typed: Kind.KW,
   Id.KW_Func: Kind.KW,
   Id.ControlFlow_Break: Kind.ControlFlow,
   Id.ControlFlow_Continue: Kind.ControlFlow,
@@ -458,6 +462,8 @@ ID_TO_KIND = {
   Id.J8_Float: Kind.J8,
   Id.J8_String: Kind.J8,
   Id.J8_Identifier: Kind.J8,
+  Id.J8_Newline: Kind.J8,
+  Id.J8_Tab: Kind.J8,
   Id.J8_LParen: Kind.J8,
   Id.J8_RParen: Kind.J8,
   Id.J8_Operator: Kind.J8,
