@@ -40,23 +40,19 @@ class cmd_value_t(pybase.CompoundObj):
 class cmd_value(object):
   class Argv(cmd_value_t):
     _type_tag = 1
-    __slots__ = ('argv', 'arg_locs', 'typed_args', 'pos_args', 'named_args',
-                 'block_arg')
+    __slots__ = ('argv', 'arg_locs', 'is_last_cmd', 'proc_args')
   
-    def __init__(self, argv, arg_locs, typed_args, pos_args, named_args,
-                 block_arg):
-      # type: (List[str], List[CompoundWord], Optional[ArgList], Optional[List[value_t]], Optional[Dict[str, value_t]], Optional[value_t]) -> None
+    def __init__(self, argv, arg_locs, is_last_cmd, proc_args):
+      # type: (List[str], List[CompoundWord], bool, Optional[ProcArgs]) -> None
       self.argv = argv
       self.arg_locs = arg_locs
-      self.typed_args = typed_args
-      self.pos_args = pos_args
-      self.named_args = named_args
-      self.block_arg = block_arg
+      self.is_last_cmd = is_last_cmd
+      self.proc_args = proc_args
   
     @staticmethod
     def CreateNull(alloc_lists=False):
       # type: () -> cmd_value.Argv
-      return cmd_value.Argv([] if alloc_lists else cast('List[str]', None), [] if alloc_lists else cast('List[CompoundWord]', None), cast('Optional[ArgList]', None), cast('Optional[List[value_t]]', None), cast('Optional[Dict[str, value_t]]', None), cast('Optional[value_t]', None))
+      return cmd_value.Argv([] if alloc_lists else cast('List[str]', None), [] if alloc_lists else cast('List[CompoundWord]', None), False, cast('Optional[ProcArgs]', None))
   
     def PrettyTree(self, trav=None):
       # type: (Optional[TraversalState]) -> hnode_t
@@ -82,29 +78,12 @@ class cmd_value(object):
           x1.children.append(h)
         L.append(Field('arg_locs', x1))
   
-      if self.typed_args is not None:  # Optional
-        x2 = self.typed_args.PrettyTree(trav=trav)
-        L.append(Field('typed_args', x2))
+      x2 = hnode.Leaf('T' if self.is_last_cmd else 'F', color_e.OtherConst)
+      L.append(Field('is_last_cmd', x2))
   
-      if self.pos_args is not None:  # List
-        x3 = hnode.Array([])
-        for i3 in self.pos_args:
-          h = (hnode.Leaf("_", color_e.OtherConst) if i3 is None else
-               i3.PrettyTree(trav=trav))
-          x3.children.append(h)
-        L.append(Field('pos_args', x3))
-  
-      if self.named_args is not None:  # Dict
-        m = hnode.Leaf("Dict", color_e.OtherConst)
-        x4 = hnode.Array([m])
-        for k4, v4 in self.named_args.iteritems():
-          x4.children.append(NewLeaf(k4, color_e.StringConst))
-          x4.children.append(v4.PrettyTree(trav=trav))
-        L.append(Field('named_args', x4))
-  
-      if self.block_arg is not None:  # Optional
-        x5 = self.block_arg.PrettyTree(trav=trav)
-        L.append(Field('block_arg', x5))
+      if self.proc_args is not None:  # Optional
+        x3 = self.proc_args.PrettyTree(trav=trav)
+        L.append(Field('proc_args', x3))
   
       return out_node
   
@@ -131,29 +110,12 @@ class cmd_value(object):
           x1.children.append(h)
         L.append(Field('arg_locs', x1))
   
-      if self.typed_args is not None:  # Optional
-        x2 = self.typed_args.AbbreviatedTree(trav=trav)
-        L.append(Field('typed_args', x2))
+      x2 = hnode.Leaf('T' if self.is_last_cmd else 'F', color_e.OtherConst)
+      L.append(Field('is_last_cmd', x2))
   
-      if self.pos_args is not None:  # List
-        x3 = hnode.Array([])
-        for i3 in self.pos_args:
-          h = (hnode.Leaf("_", color_e.OtherConst) if i3 is None else
-               i3.AbbreviatedTree(trav=trav))
-          x3.children.append(h)
-        L.append(Field('pos_args', x3))
-  
-      if self.named_args is not None:  # Dict
-        m = hnode.Leaf("Dict", color_e.OtherConst)
-        x4 = hnode.Array([m])
-        for k4, v4 in self.named_args.iteritems():
-          x4.children.append(NewLeaf(k4, color_e.StringConst))
-          x4.children.append(v4.AbbreviatedTree(trav=trav))
-        L.append(Field('named_args', x4))
-  
-      if self.block_arg is not None:  # Optional
-        x5 = self.block_arg.AbbreviatedTree(trav=trav)
-        L.append(Field('block_arg', x5))
+      if self.proc_args is not None:  # Optional
+        x3 = self.proc_args.AbbreviatedTree(trav=trav)
+        L.append(Field('proc_args', x3))
   
       return out_node
   
@@ -257,14 +219,14 @@ class cmd_value(object):
   pass
 
 class part_value_e(object):
-  String = 65
+  String = 66
   Array = 2
   ExtGlob = 3
 
 _part_value_str = {
   2: 'Array',
   3: 'ExtGlob',
-  65: 'String',
+  66: 'String',
 }
 
 def part_value_str(tag, dot=True):
@@ -1540,8 +1502,99 @@ class AssignArg(pybase.CompoundObj):
     # type: (Optional[TraversalState]) -> hnode_t
     return self._AbbreviatedTree(trav=trav)
 
-class Piece(part_value_t):
+class ProcArgs(pybase.CompoundObj):
   _type_tag = 65
+  __slots__ = ('typed_args', 'pos_args', 'named_args', 'block_arg')
+
+  def __init__(self, typed_args, pos_args, named_args, block_arg):
+    # type: (ArgList, Optional[List[value_t]], Optional[Dict[str, value_t]], Optional[value_t]) -> None
+    self.typed_args = typed_args
+    self.pos_args = pos_args
+    self.named_args = named_args
+    self.block_arg = block_arg
+
+  @staticmethod
+  def CreateNull(alloc_lists=False):
+    # type: () -> ProcArgs
+    return ProcArgs(cast(ArgList, None), cast('Optional[List[value_t]]', None), cast('Optional[Dict[str, value_t]]', None), cast('Optional[value_t]', None))
+
+  def PrettyTree(self, trav=None):
+    # type: (Optional[TraversalState]) -> hnode_t
+    trav = trav or TraversalState()
+    heap_id = id(self)
+    if heap_id in trav.seen:
+      return hnode.AlreadySeen(heap_id)
+    trav.seen[heap_id] = True
+    out_node = NewRecord('ProcArgs')
+    L = out_node.fields
+
+    assert self.typed_args is not None
+    x0 = self.typed_args.PrettyTree(trav=trav)
+    L.append(Field('typed_args', x0))
+
+    if self.pos_args is not None:  # List
+      x1 = hnode.Array([])
+      for i1 in self.pos_args:
+        h = (hnode.Leaf("_", color_e.OtherConst) if i1 is None else
+             i1.PrettyTree(trav=trav))
+        x1.children.append(h)
+      L.append(Field('pos_args', x1))
+
+    if self.named_args is not None:  # Dict
+      m = hnode.Leaf("Dict", color_e.OtherConst)
+      x2 = hnode.Array([m])
+      for k2, v2 in self.named_args.iteritems():
+        x2.children.append(NewLeaf(k2, color_e.StringConst))
+        x2.children.append(v2.PrettyTree(trav=trav))
+      L.append(Field('named_args', x2))
+
+    if self.block_arg is not None:  # Optional
+      x3 = self.block_arg.PrettyTree(trav=trav)
+      L.append(Field('block_arg', x3))
+
+    return out_node
+
+  def _AbbreviatedTree(self, trav=None):
+    # type: (Optional[TraversalState]) -> hnode_t
+    trav = trav or TraversalState()
+    heap_id = id(self)
+    if heap_id in trav.seen:
+      return hnode.AlreadySeen(heap_id)
+    trav.seen[heap_id] = True
+    out_node = NewRecord('ProcArgs')
+    L = out_node.fields
+    assert self.typed_args is not None
+    x0 = self.typed_args.AbbreviatedTree(trav=trav)
+    L.append(Field('typed_args', x0))
+
+    if self.pos_args is not None:  # List
+      x1 = hnode.Array([])
+      for i1 in self.pos_args:
+        h = (hnode.Leaf("_", color_e.OtherConst) if i1 is None else
+             i1.AbbreviatedTree(trav=trav))
+        x1.children.append(h)
+      L.append(Field('pos_args', x1))
+
+    if self.named_args is not None:  # Dict
+      m = hnode.Leaf("Dict", color_e.OtherConst)
+      x2 = hnode.Array([m])
+      for k2, v2 in self.named_args.iteritems():
+        x2.children.append(NewLeaf(k2, color_e.StringConst))
+        x2.children.append(v2.AbbreviatedTree(trav=trav))
+      L.append(Field('named_args', x2))
+
+    if self.block_arg is not None:  # Optional
+      x3 = self.block_arg.AbbreviatedTree(trav=trav)
+      L.append(Field('block_arg', x3))
+
+    return out_node
+
+  def AbbreviatedTree(self, trav=None):
+    # type: (Optional[TraversalState]) -> hnode_t
+    return self._AbbreviatedTree(trav=trav)
+
+class Piece(part_value_t):
+  _type_tag = 66
   __slots__ = ('s', 'quoted', 'do_split')
 
   def __init__(self, s, quoted, do_split):
@@ -1601,7 +1654,7 @@ class Piece(part_value_t):
     return self._AbbreviatedTree(trav=trav)
 
 class VarSubState(pybase.CompoundObj):
-  _type_tag = 66
+  _type_tag = 67
   __slots__ = ('join_array', 'is_type_query', 'has_test_op')
 
   def __init__(self, join_array, is_type_query, has_test_op):
@@ -1661,7 +1714,7 @@ class VarSubState(pybase.CompoundObj):
     return self._AbbreviatedTree(trav=trav)
 
 class Cell(pybase.CompoundObj):
-  _type_tag = 67
+  _type_tag = 68
   __slots__ = ('exported', 'readonly', 'nameref', 'val')
 
   def __init__(self, exported, readonly, nameref, val):
@@ -1730,7 +1783,7 @@ class Cell(pybase.CompoundObj):
     return self._AbbreviatedTree(trav=trav)
 
 class VTestPlace(pybase.CompoundObj):
-  _type_tag = 68
+  _type_tag = 69
   __slots__ = ('name', 'index')
 
   def __init__(self, name, index):
@@ -1787,7 +1840,7 @@ class VTestPlace(pybase.CompoundObj):
     return self._AbbreviatedTree(trav=trav)
 
 class RedirValue(pybase.CompoundObj):
-  _type_tag = 69
+  _type_tag = 70
   __slots__ = ('op_id', 'op_loc', 'loc', 'arg')
 
   def __init__(self, op_id, op_loc, loc, arg):
@@ -1860,7 +1913,7 @@ class RedirValue(pybase.CompoundObj):
     return self._AbbreviatedTree(trav=trav)
 
 class StatusArray(pybase.CompoundObj):
-  _type_tag = 70
+  _type_tag = 71
   __slots__ = ('codes', 'locs')
 
   def __init__(self, codes, locs):
@@ -1929,7 +1982,7 @@ class StatusArray(pybase.CompoundObj):
     return self._AbbreviatedTree(trav=trav)
 
 class CommandStatus(pybase.CompoundObj):
-  _type_tag = 71
+  _type_tag = 72
   __slots__ = ('check_errexit', 'show_code', 'pipe_negated', 'pipe_status',
                'pipe_locs')
 
@@ -2021,7 +2074,7 @@ class CommandStatus(pybase.CompoundObj):
     return self._AbbreviatedTree(trav=trav)
 
 class HayNode(pybase.CompoundObj):
-  _type_tag = 72
+  _type_tag = 73
   __slots__ = ('children',)
 
   def __init__(self, children):
