@@ -459,58 +459,62 @@ class regex_match(object):
   pass
 
 class value_e(object):
-  Undef = 1
-  Str = 2
-  BashArray = 3
-  BashAssoc = 4
-  Null = 5
-  Bool = 6
-  Int = 7
-  Float = 8
-  List = 9
-  Dict = 10
-  Eggex = 11
+  Interrupted = 1
+  Stdin = 2
+  Undef = 3
+  Str = 4
+  BashArray = 5
+  SparseArray = 6
+  BashAssoc = 7
+  Null = 8
+  Bool = 9
+  Int = 10
+  Float = 11
+  List = 12
+  Dict = 13
+  Obj = 68
+  Eggex = 15
   Match = 67
-  Expr = 13
-  Command = 14
-  Block = 15
-  Place = 16
-  Module = 17
-  IO = 18
-  Guts = 19
-  BuiltinFunc = 20
-  BoundFunc = 21
-  Proc = 22
-  Func = 23
-  Slice = 24
-  Range = 25
+  Expr = 17
+  Command = 18
+  Block = 19
+  Place = 20
+  Module = 21
+  BuiltinFunc = 22
+  BoundFunc = 23
+  Proc = 24
+  Func = 25
+  Range = 26
+  Slice = 27
 
 _value_str = {
-  1: 'Undef',
-  2: 'Str',
-  3: 'BashArray',
-  4: 'BashAssoc',
-  5: 'Null',
-  6: 'Bool',
-  7: 'Int',
-  8: 'Float',
-  9: 'List',
-  10: 'Dict',
-  11: 'Eggex',
-  13: 'Expr',
-  14: 'Command',
-  15: 'Block',
-  16: 'Place',
-  17: 'Module',
-  18: 'IO',
-  19: 'Guts',
-  20: 'BuiltinFunc',
-  21: 'BoundFunc',
-  22: 'Proc',
-  23: 'Func',
-  24: 'Slice',
-  25: 'Range',
+  1: 'Interrupted',
+  2: 'Stdin',
+  3: 'Undef',
+  4: 'Str',
+  5: 'BashArray',
+  6: 'SparseArray',
+  7: 'BashAssoc',
+  8: 'Null',
+  9: 'Bool',
+  10: 'Int',
+  11: 'Float',
+  12: 'List',
+  13: 'Dict',
+  15: 'Eggex',
+  17: 'Expr',
+  18: 'Command',
+  19: 'Block',
+  20: 'Place',
+  21: 'Module',
+  22: 'BuiltinFunc',
+  23: 'BoundFunc',
+  24: 'Proc',
+  25: 'Func',
+  26: 'Range',
+  27: 'Slice',
   67: 'Match',
+  68: 'Obj',
 }
 
 def value_str(tag, dot=True):
@@ -527,10 +531,80 @@ class value_t(pybase.CompoundObj):
     return self._type_tag
   pass
 
-class value__Undef(value_t):
+class value__Interrupted(value_t):
   _type_tag = 1
   __slots__ = ()
 
+  def __init__(self, ):
+    # type: () -> None
+    pass
+
+  def PrettyTree(self, trav=None):
+    # type: (Optional[TraversalState]) -> hnode_t
+    trav = trav or TraversalState()
+    heap_id = id(self)
+    if heap_id in trav.seen:
+      return hnode.AlreadySeen(heap_id)
+    trav.seen[heap_id] = True
+    out_node = NewRecord('value__Interrupted')
+    L = out_node.fields
+
+    return out_node
+
+  def _AbbreviatedTree(self, trav=None):
+    # type: (Optional[TraversalState]) -> hnode_t
+    trav = trav or TraversalState()
+    heap_id = id(self)
+    if heap_id in trav.seen:
+      return hnode.AlreadySeen(heap_id)
+    trav.seen[heap_id] = True
+    out_node = NewRecord('value__Interrupted')
+    L = out_node.fields
+    return out_node
+
+  def AbbreviatedTree(self, trav=None):
+    # type: (Optional[TraversalState]) -> hnode_t
+    return self._AbbreviatedTree(trav=trav)
+
+class value__Stdin(value_t):
+  _type_tag = 2
+  __slots__ = ()
+
+  def __init__(self, ):
+    # type: () -> None
+    pass
+
+  def PrettyTree(self, trav=None):
+    # type: (Optional[TraversalState]) -> hnode_t
+    trav = trav or TraversalState()
+    heap_id = id(self)
+    if heap_id in trav.seen:
+      return hnode.AlreadySeen(heap_id)
+    trav.seen[heap_id] = True
+    out_node = NewRecord('value__Stdin')
+    L = out_node.fields
+
+    return out_node
+
+  def _AbbreviatedTree(self, trav=None):
+    # type: (Optional[TraversalState]) -> hnode_t
+    trav = trav or TraversalState()
+    heap_id = id(self)
+    if heap_id in trav.seen:
+      return hnode.AlreadySeen(heap_id)
+    trav.seen[heap_id] = True
+    out_node = NewRecord('value__Stdin')
+    L = out_node.fields
+    return out_node
+
+  def AbbreviatedTree(self, trav=None):
+    # type: (Optional[TraversalState]) -> hnode_t
+    return self._AbbreviatedTree(trav=trav)
+
+class value__Undef(value_t):
+  _type_tag = 3
+  __slots__ = ()
+
   def __init__(self, ):
     # type: () -> None
     pass
@@ -563,7 +637,7 @@ class value__Undef(value_t):
     return self._AbbreviatedTree(trav=trav)
 
 class value__Null(value_t):
-  _type_tag = 5
+  _type_tag = 8
   __slots__ = ()
 
   def __init__(self, ):
@@ -598,10 +672,14 @@ class value__Null(value_t):
     return self._AbbreviatedTree(trav=trav)
 
 class value(object):
+  Interrupted = value__Interrupted()
+  
+  Stdin = value__Stdin()
+  
   Undef = value__Undef()
   
   class Str(value_t):
-    _type_tag = 2
+    _type_tag = 4
     __slots__ = ('s',)
   
     def __init__(self, s):
@@ -647,7 +725,7 @@ class value(object):
       return self._AbbreviatedTree(trav=trav)
   
   class BashArray(value_t):
-    _type_tag = 3
+    _type_tag = 5
     __slots__ = ('strs',)
   
     def __init__(self, strs):
@@ -698,8 +776,71 @@ class value(object):
       # type: (Optional[TraversalState]) -> hnode_t
       return self._AbbreviatedTree(trav=trav)
   
+  class SparseArray(value_t):
+    _type_tag = 6
+    __slots__ = ('d', 'max_index')
+  
+    def __init__(self, d, max_index):
+      # type: (Dict[mops.BigInt, str], mops.BigInt) -> None
+      self.d = d
+      self.max_index = max_index
+  
+    @staticmethod
+    def CreateNull(alloc_lists=False):
+      # type: () -> value.SparseArray
+      return value.SparseArray(cast('Dict[mops.BigInt, str]', None), mops.BigInt(-1))
+  
+    def PrettyTree(self, trav=None):
+      # type: (Optional[TraversalState]) -> hnode_t
+      trav = trav or TraversalState()
+      heap_id = id(self)
+      if heap_id in trav.seen:
+        return hnode.AlreadySeen(heap_id)
+      trav.seen[heap_id] = True
+      out_node = NewRecord('value.SparseArray')
+      L = out_node.fields
+  
+      if self.d is not None:  # Dict
+        m = hnode.Leaf("Dict", color_e.OtherConst)
+        x0 = hnode.Array([m])
+        for k0, v0 in self.d.iteritems():
+          x0.children.append(hnode.Leaf(mops.ToStr(k0), color_e.OtherConst))
+          x0.children.append(NewLeaf(v0, color_e.StringConst))
+        L.append(Field('d', x0))
+  
+      x1 = hnode.Leaf(mops.ToStr(self.max_index), color_e.OtherConst)
+      L.append(Field('max_index', x1))
+  
+      return out_node
+  
+    def _AbbreviatedTree(self, trav=None):
+      # type: (Optional[TraversalState]) -> hnode_t
+      trav = trav or TraversalState()
+      heap_id = id(self)
+      if heap_id in trav.seen:
+        return hnode.AlreadySeen(heap_id)
+      trav.seen[heap_id] = True
+      out_node = NewRecord('value.SparseArray')
+      L = out_node.fields
+      if self.d is not None:  # Dict
+        m = hnode.Leaf("Dict", color_e.OtherConst)
+        x0 = hnode.Array([m])
+        for k0, v0 in self.d.iteritems():
+          x0.children.append(hnode.Leaf(mops.ToStr(k0), color_e.OtherConst))
+          x0.children.append(NewLeaf(v0, color_e.StringConst))
+        L.append(Field('d', x0))
+  
+      x1 = hnode.Leaf(mops.ToStr(self.max_index), color_e.OtherConst)
+      L.append(Field('max_index', x1))
+  
+      return out_node
+  
+    def AbbreviatedTree(self, trav=None):
+      # type: (Optional[TraversalState]) -> hnode_t
+      return self._AbbreviatedTree(trav=trav)
+  
   class BashAssoc(value_t):
-    _type_tag = 4
+    _type_tag = 7
     __slots__ = ('d',)
   
     def __init__(self, d):
@@ -757,7 +898,7 @@ class value(object):
   Null = value__Null()
   
   class Bool(value_t):
-    _type_tag = 6
+    _type_tag = 9
     __slots__ = ('b',)
   
     def __init__(self, b):
@@ -803,7 +944,7 @@ class value(object):
       return self._AbbreviatedTree(trav=trav)
   
   class Int(value_t):
-    _type_tag = 7
+    _type_tag = 10
     __slots__ = ('i',)
   
     def __init__(self, i):
@@ -849,7 +990,7 @@ class value(object):
       return self._AbbreviatedTree(trav=trav)
   
   class Float(value_t):
-    _type_tag = 8
+    _type_tag = 11
     __slots__ = ('f',)
   
     def __init__(self, f):
@@ -895,7 +1036,7 @@ class value(object):
       return self._AbbreviatedTree(trav=trav)
   
   class List(value_t):
-    _type_tag = 9
+    _type_tag = 12
     __slots__ = ('items',)
   
     def __init__(self, items):
@@ -951,7 +1092,7 @@ class value(object):
       return self._AbbreviatedTree(trav=trav)
   
   class Dict(value_t):
-    _type_tag = 10
+    _type_tag = 13
     __slots__ = ('d',)
   
     def __init__(self, d):
@@ -1007,7 +1148,7 @@ class value(object):
       return self._AbbreviatedTree(trav=trav)
   
   class Eggex(value_t):
-    _type_tag = 11
+    _type_tag = 15
     __slots__ = ('spliced', 'canonical_flags', 'convert_funcs', 'convert_toks',
                  'as_ere', 'capture_names')
   
@@ -1120,7 +1261,7 @@ class value(object):
       return self._AbbreviatedTree(trav=trav)
   
   class Expr(value_t):
-    _type_tag = 13
+    _type_tag = 17
     __slots__ = ('e',)
   
     def __init__(self, e):
@@ -1168,7 +1309,7 @@ class value(object):
       return self._AbbreviatedTree(trav=trav)
   
   class Command(value_t):
-    _type_tag = 14
+    _type_tag = 18
     __slots__ = ('c',)
   
     def __init__(self, c):
@@ -1216,7 +1357,7 @@ class value(object):
       return self._AbbreviatedTree(trav=trav)
   
   class Block(value_t):
-    _type_tag = 15
+    _type_tag = 19
     __slots__ = ('block',)
   
     def __init__(self, block):
@@ -1264,7 +1405,7 @@ class value(object):
       return self._AbbreviatedTree(trav=trav)
   
   class Place(value_t):
-    _type_tag = 16
+    _type_tag = 20
     __slots__ = ('lval', 'frame')
   
     def __init__(self, lval, frame):
@@ -1329,7 +1470,7 @@ class value(object):
       return self._AbbreviatedTree(trav=trav)
   
   class Module(value_t):
-    _type_tag = 17
+    _type_tag = 21
     __slots__ = ('defs',)
   
     def __init__(self, defs):
@@ -1384,107 +1525,8 @@ class value(object):
       # type: (Optional[TraversalState]) -> hnode_t
       return self._AbbreviatedTree(trav=trav)
   
-  class IO(value_t):
-    _type_tag = 18
-    __slots__ = ('cmd_ev', 'prompt_ev')
-  
-    def __init__(self, cmd_ev, prompt_ev):
-      # type: (Any, Any) -> None
-      self.cmd_ev = cmd_ev
-      self.prompt_ev = prompt_ev
-  
-    @staticmethod
-    def CreateNull(alloc_lists=False):
-      # type: () -> value.IO
-      return value.IO(cast(Any, None), cast(Any, None))
-  
-    def PrettyTree(self, trav=None):
-      # type: (Optional[TraversalState]) -> hnode_t
-      trav = trav or TraversalState()
-      heap_id = id(self)
-      if heap_id in trav.seen:
-        return hnode.AlreadySeen(heap_id)
-      trav.seen[heap_id] = True
-      out_node = NewRecord('value.IO')
-      L = out_node.fields
-  
-      x0 = hnode.External(self.cmd_ev)
-      L.append(Field('cmd_ev', x0))
-  
-      x1 = hnode.External(self.prompt_ev)
-      L.append(Field('prompt_ev', x1))
-  
-      return out_node
-  
-    def _AbbreviatedTree(self, trav=None):
-      # type: (Optional[TraversalState]) -> hnode_t
-      trav = trav or TraversalState()
-      heap_id = id(self)
-      if heap_id in trav.seen:
-        return hnode.AlreadySeen(heap_id)
-      trav.seen[heap_id] = True
-      out_node = NewRecord('value.IO')
-      L = out_node.fields
-      x0 = hnode.External(self.cmd_ev)
-      L.append(Field('cmd_ev', x0))
-  
-      x1 = hnode.External(self.prompt_ev)
-      L.append(Field('prompt_ev', x1))
-  
-      return out_node
-  
-    def AbbreviatedTree(self, trav=None):
-      # type: (Optional[TraversalState]) -> hnode_t
-      return self._AbbreviatedTree(trav=trav)
-  
-  class Guts(value_t):
-    _type_tag = 19
-    __slots__ = ('vm',)
-  
-    def __init__(self, vm):
-      # type: (Any) -> None
-      self.vm = vm
-  
-    @staticmethod
-    def CreateNull(alloc_lists=False):
-      # type: () -> value.Guts
-      return value.Guts(cast(Any, None))
-  
-    def PrettyTree(self, trav=None):
-      # type: (Optional[TraversalState]) -> hnode_t
-      trav = trav or TraversalState()
-      heap_id = id(self)
-      if heap_id in trav.seen:
-        return hnode.AlreadySeen(heap_id)
-      trav.seen[heap_id] = True
-      out_node = NewRecord('value.Guts')
-      L = out_node.fields
-  
-      x0 = hnode.External(self.vm)
-      L.append(Field('vm', x0))
-  
-      return out_node
-  
-    def _AbbreviatedTree(self, trav=None):
-      # type: (Optional[TraversalState]) -> hnode_t
-      trav = trav or TraversalState()
-      heap_id = id(self)
-      if heap_id in trav.seen:
-        return hnode.AlreadySeen(heap_id)
-      trav.seen[heap_id] = True
-      out_node = NewRecord('value.Guts')
-      L = out_node.fields
-      x0 = hnode.External(self.vm)
-      L.append(Field('vm', x0))
-  
-      return out_node
-  
-    def AbbreviatedTree(self, trav=None):
-      # type: (Optional[TraversalState]) -> hnode_t
-      return self._AbbreviatedTree(trav=trav)
-  
   class BuiltinFunc(value_t):
-    _type_tag = 20
+    _type_tag = 22
     __slots__ = ('callable',)
   
     def __init__(self, callable):
@@ -1530,7 +1572,7 @@ class value(object):
       return self._AbbreviatedTree(trav=trav)
   
   class BoundFunc(value_t):
-    _type_tag = 21
+    _type_tag = 23
     __slots__ = ('me', 'func')
   
     def __init__(self, me, func):
@@ -1587,7 +1629,7 @@ class value(object):
       return self._AbbreviatedTree(trav=trav)
   
   class Proc(value_t):
-    _type_tag = 22
+    _type_tag = 24
     __slots__ = ('name', 'name_tok', 'sig', 'body', 'defaults', 'sh_compat')
   
     def __init__(self, name, name_tok, sig, body, defaults, sh_compat):
@@ -1676,7 +1718,7 @@ class value(object):
       return self._AbbreviatedTree(trav=trav)
   
   class Func(value_t):
-    _type_tag = 23
+    _type_tag = 25
     __slots__ = ('name', 'parsed', 'pos_defaults', 'named_defaults', 'module_')
   
     def __init__(self, name, parsed, pos_defaults, named_defaults, module_):
@@ -1781,8 +1823,61 @@ class value(object):
       # type: (Optional[TraversalState]) -> hnode_t
       return self._AbbreviatedTree(trav=trav)
   
+  class Range(value_t):
+    _type_tag = 26
+    __slots__ = ('lower', 'upper')
+  
+    def __init__(self, lower, upper):
+      # type: (int, int) -> None
+      self.lower = lower
+      self.upper = upper
+  
+    @staticmethod
+    def CreateNull(alloc_lists=False):
+      # type: () -> value.Range
+      return value.Range(-1, -1)
+  
+    def PrettyTree(self, trav=None):
+      # type: (Optional[TraversalState]) -> hnode_t
+      trav = trav or TraversalState()
+      heap_id = id(self)
+      if heap_id in trav.seen:
+        return hnode.AlreadySeen(heap_id)
+      trav.seen[heap_id] = True
+      out_node = NewRecord('value.Range')
+      L = out_node.fields
+  
+      x0 = hnode.Leaf(str(self.lower), color_e.OtherConst)
+      L.append(Field('lower', x0))
+  
+      x1 = hnode.Leaf(str(self.upper), color_e.OtherConst)
+      L.append(Field('upper', x1))
+  
+      return out_node
+  
+    def _AbbreviatedTree(self, trav=None):
+      # type: (Optional[TraversalState]) -> hnode_t
+      trav = trav or TraversalState()
+      heap_id = id(self)
+      if heap_id in trav.seen:
+        return hnode.AlreadySeen(heap_id)
+      trav.seen[heap_id] = True
+      out_node = NewRecord('value.Range')
+      L = out_node.fields
+      x0 = hnode.Leaf(str(self.lower), color_e.OtherConst)
+      L.append(Field('lower', x0))
+  
+      x1 = hnode.Leaf(str(self.upper), color_e.OtherConst)
+      L.append(Field('upper', x1))
+  
+      return out_node
+  
+    def AbbreviatedTree(self, trav=None):
+      # type: (Optional[TraversalState]) -> hnode_t
+      return self._AbbreviatedTree(trav=trav)
+  
   class Slice(value_t):
-    _type_tag = 24
+    _type_tag = 27
     __slots__ = ('lower', 'upper')
   
     def __init__(self, lower, upper):
@@ -1838,59 +1933,6 @@ class value(object):
       # type: (Optional[TraversalState]) -> hnode_t
       return self._AbbreviatedTree(trav=trav)
   
-  class Range(value_t):
-    _type_tag = 25
-    __slots__ = ('lower', 'upper')
-  
-    def __init__(self, lower, upper):
-      # type: (int, int) -> None
-      self.lower = lower
-      self.upper = upper
-  
-    @staticmethod
-    def CreateNull(alloc_lists=False):
-      # type: () -> value.Range
-      return value.Range(-1, -1)
-  
-    def PrettyTree(self, trav=None):
-      # type: (Optional[TraversalState]) -> hnode_t
-      trav = trav or TraversalState()
-      heap_id = id(self)
-      if heap_id in trav.seen:
-        return hnode.AlreadySeen(heap_id)
-      trav.seen[heap_id] = True
-      out_node = NewRecord('value.Range')
-      L = out_node.fields
-  
-      x0 = hnode.Leaf(str(self.lower), color_e.OtherConst)
-      L.append(Field('lower', x0))
-  
-      x1 = hnode.Leaf(str(self.upper), color_e.OtherConst)
-      L.append(Field('upper', x1))
-  
-      return out_node
-  
-    def _AbbreviatedTree(self, trav=None):
-      # type: (Optional[TraversalState]) -> hnode_t
-      trav = trav or TraversalState()
-      heap_id = id(self)
-      if heap_id in trav.seen:
-        return hnode.AlreadySeen(heap_id)
-      trav.seen[heap_id] = True
-      out_node = NewRecord('value.Range')
-      L = out_node.fields
-      x0 = hnode.Leaf(str(self.lower), color_e.OtherConst)
-      L.append(Field('lower', x0))
-  
-      x1 = hnode.Leaf(str(self.upper), color_e.OtherConst)
-      L.append(Field('upper', x1))
-  
-      return out_node
-  
-    def AbbreviatedTree(self, trav=None):
-      # type: (Optional[TraversalState]) -> hnode_t
-      return self._AbbreviatedTree(trav=trav)
-  
   pass
 
 class IntBox(pybase.CompoundObj):
@@ -2161,3 +2203,68 @@ class RegexMatch(regex_match_t, value_t):
     # type: (Optional[TraversalState]) -> hnode_t
     return self._AbbreviatedTree(trav=trav)
 
+class Obj(value_t):
+  _type_tag = 68
+  __slots__ = ('prototype', 'd')
+
+  def __init__(self, prototype, d):
+    # type: (Optional[Obj], Dict[str, value_t]) -> None
+    self.prototype = prototype
+    self.d = d
+
+  @staticmethod
+  def CreateNull(alloc_lists=False):
+    # type: () -> Obj
+    return Obj(cast('Optional[Obj]', None), cast('Dict[str, value_t]', None))
+
+  def PrettyTree(self, trav=None):
+    # type: (Optional[TraversalState]) -> hnode_t
+    trav = trav or TraversalState()
+    heap_id = id(self)
+    if heap_id in trav.seen:
+      return hnode.AlreadySeen(heap_id)
+    trav.seen[heap_id] = True
+    out_node = NewRecord('Obj')
+    L = out_node.fields
+
+    if self.prototype is not None:  # Optional
+      x0 = self.prototype.PrettyTree(trav=trav)
+      L.append(Field('prototype', x0))
+
+    if self.d is not None:  # Dict
+      m = hnode.Leaf("Dict", color_e.OtherConst)
+      x1 = hnode.Array([m])
+      for k1, v1 in self.d.iteritems():
+        x1.children.append(NewLeaf(k1, color_e.StringConst))
+        x1.children.append(v1.PrettyTree(trav=trav))
+      L.append(Field('d', x1))
+
+    return out_node
+
+  def _AbbreviatedTree(self, trav=None):
+    # type: (Optional[TraversalState]) -> hnode_t
+    trav = trav or TraversalState()
+    heap_id = id(self)
+    if heap_id in trav.seen:
+      return hnode.AlreadySeen(heap_id)
+    trav.seen[heap_id] = True
+    out_node = NewRecord('Obj')
+    L = out_node.fields
+    if self.prototype is not None:  # Optional
+      x0 = self.prototype.AbbreviatedTree(trav=trav)
+      L.append(Field('prototype', x0))
+
+    if self.d is not None:  # Dict
+      m = hnode.Leaf("Dict", color_e.OtherConst)
+      x1 = hnode.Array([m])
+      for k1, v1 in self.d.iteritems():
+        x1.children.append(NewLeaf(k1, color_e.StringConst))
+        x1.children.append(v1.AbbreviatedTree(trav=trav))
+      L.append(Field('d', x1))
+
+    return out_node
+
+  def AbbreviatedTree(self, trav=None):
+    # type: (Optional[TraversalState]) -> hnode_t
+    return self._AbbreviatedTree(trav=trav)
+
