@@ -12,6 +12,8 @@ interface?
 """
 from __future__ import print_function
 
+from typing import Tuple
+
 
 class BigInt(object):
 
@@ -19,10 +21,18 @@ class BigInt(object):
         # type: (int) -> None
         self.i = i
 
-    # Prevent possible mistakes.  Could do this with other operators
     def __eq__(self, other):
         # type: (object) -> bool
-        raise AssertionError('Use mops.Equal()')
+
+        # Disabled check
+        # Prevent possible mistakes.  Could do this with other operators
+        # raise AssertionError('Use mops.Equal()')
+
+        if not isinstance(other, BigInt):
+            raise AssertionError()
+
+        # Used for hashing
+        return self.i == other.i
 
     def __gt__(self, other):
         # type: (object) -> bool
@@ -32,10 +42,16 @@ class BigInt(object):
         # type: (object) -> bool
         raise AssertionError('Use functions in mops.py')
 
+    def __hash__(self):
+        # type: () -> int
+        """For dict lookups."""
+        return hash(self.i)
+
 
 ZERO = BigInt(0)
 ONE = BigInt(1)
 MINUS_ONE = BigInt(-1)
+MINUS_TWO = BigInt(-2)  # for printf
 
 
 def ToStr(b):
@@ -43,6 +59,21 @@ def ToStr(b):
     return str(b.i)
 
 
+def ToOctal(b):
+    # type: (BigInt) -> str
+    return '%o' % b.i
+
+
+def ToHexUpper(b):
+    # type: (BigInt) -> str
+    return '%X' % b.i
+
+
+def ToHexLower(b):
+    # type: (BigInt) -> str
+    return '%x' % b.i
+
+
 def FromStr(s, base=10):
     # type: (str, int) -> BigInt
     return BigInt(int(s, base))
@@ -79,9 +110,15 @@ def ToFloat(b):
 
 
 def FromFloat(f):
-    # type: (float) -> BigInt
+    # type: (float) -> Tuple[bool, BigInt]
     """Used by int(3.14) in Oils"""
-    return BigInt(int(f))
+    try:
+        big = int(f)
+    except ValueError:  # NAN
+        return False, MINUS_ONE
+    except OverflowError:  # INFINITY
+        return False, MINUS_ONE
+    return True, BigInt(big)
 
 
 # Can't use operator overloading
@@ -109,23 +146,52 @@ def Mul(a, b):
 
 def Div(a, b):
     # type: (BigInt, BigInt) -> BigInt
-    """
-    Divide, for positive integers only
+    """Integer division.
 
-    Question: does Oils behave like C remainder when it's positive?  Then we
-    could be more efficient with a different layering?
+    Oils rounds toward zero.
+
+    Python rounds toward negative infinity, while C++ rounds toward zero.  We
+    have to work around Python a bit.
     """
-    assert a.i >= 0 and b.i >= 0, (a.i, b.i)
-    return BigInt(a.i // b.i)
+    assert b.i != 0, b.i  # divisor can't be zero -- caller checks
+
+    # Only use Python // on non-negative numbers.  Apply sign afterward.
+    sign = 1
+
+    if a.i < 0:
+        pa = -a.i
+        sign = -1
+    else:
+        pa = a.i
+
+    if b.i < 0:
+        pb = -b.i
+        sign = -sign
+    else:
+        pb = b.i
+
+    return BigInt(sign * (pa // pb))
 
 
 def Rem(a, b):
     # type: (BigInt, BigInt) -> BigInt
-    """
-    Remainder, for positive integers only
-    """
-    assert a.i >= 0 and b.i >= 0, (a.i, b.i)
-    return BigInt(a.i % b.i)
+    """Integer remainder."""
+    assert b.i != 0, b.i  # YSH divisor must be positive, but OSH can be negative
+
+    # Only use Python % on non-negative numbers.  Apply sign afterward.
+    if a.i < 0:
+        pa = -a.i
+        sign = -1
+    else:
+        pa = a.i
+        sign = 1
+
+    if b.i < 0:
+        pb = -b.i
+    else:
+        pb = b.i
+
+    return BigInt(sign * (pa % pb))
 
 
 def Equal(a, b):
@@ -143,14 +209,13 @@ def Greater(a, b):
 
 def LShift(a, b):
     # type: (BigInt, BigInt) -> BigInt
-    """
-    Any semantic issues here?  Signed left shift
-    """
+    assert b.i >= 0, b.i  # Must be checked by caller
     return BigInt(a.i << b.i)
 
 
 def RShift(a, b):
     # type: (BigInt, BigInt) -> BigInt
+    assert b.i >= 0, b.i  # Must be checked by caller
     return BigInt(a.i >> b.i)
 
 
