@@ -9,13 +9,13 @@ import time as time_
 
 from _devbuild.gen.id_kind_asdl import Id, Id_t
 from _devbuild.gen.syntax_asdl import (loc, command_t, source, CompoundWord)
-from _devbuild.gen.value_asdl import (value, value_e, value_t)
+from _devbuild.gen.value_asdl import (value, value_e, value_t, Obj)
 from core import alloc
 from core import main_loop
 from core import error
 from core import pyos
 from core import state
-from core import ui
+from display import ui
 from frontend import consts
 from frontend import match
 from frontend import reader
@@ -104,7 +104,7 @@ class Evaluator(object):
         # type: (str, str, ParseContext, Mem) -> None
         self.word_ev = None  # type: word_eval.AbstractWordEvaluator
         self.expr_ev = None  # type: expr_eval.ExprEvaluator
-        self.global_io = None  # type: value.IO
+        self.global_io = None  # type: Obj
 
         assert lang in ('osh', 'ysh'), lang
         self.lang = lang
@@ -370,4 +370,4 @@ class UserPlugin(object):
         # Save this so PROMPT_COMMAND can't set $?
         with state.ctx_Registers(self.mem):
             # Catches fatal execution error
-            self.cmd_ev.ExecuteAndCatch(node)
+            self.cmd_ev.ExecuteAndCatch(node, 0)
