@@ -49,7 +49,8 @@ from core import error
 from core.error import e_die, e_die_status, e_strict, e_usage
 from core import num
 from core import state
-from core import ui
+from display import ui
+from core import util
 from frontend import consts
 from frontend import lexer
 from frontend import location
@@ -68,7 +69,6 @@ from libc import FNM_CASEFOLD, REG_ICASE
 
 from typing import Tuple, Optional, cast, TYPE_CHECKING
 if TYPE_CHECKING:
-    from core.ui import ErrorFormatter
     from core import optview
 
 _ = log
@@ -309,7 +309,7 @@ class ArithEvaluator(object):
             exec_opts,  # type: optview.Exec
             mutable_opts,  # type: state.MutableOpts
             parse_ctx,  # type: Optional[parse_lib.ParseContext]
-            errfmt,  # type: ErrorFormatter
+            errfmt,  # type: ui.ErrorFormatter
     ):
         # type: (...) -> None
         self.word_ev = None  # type: word_eval.StringWordEvaluator
@@ -545,7 +545,13 @@ class ArithEvaluator(object):
 
         UP_node = node
         with tagswitch(node) as case:
-            if case(arith_expr_e.VarSub):  # $(( x ))  (can be array)
+            if case(arith_expr_e.EmptyZero):  # $(( ))
+                return value.Int(mops.ZERO)  # Weird axiom
+
+            elif case(arith_expr_e.EmptyOne):  # for (( ; ; ))
+                return value.Int(mops.ONE)
+
+            elif case(arith_expr_e.VarSub):  # $(( x ))  (can be array)
                 vsub = cast(Token, UP_node)
                 var_name = lexer.LazyStr(vsub)
                 val = self.mem.GetValue(var_name)
@@ -611,12 +617,12 @@ class ArithEvaluator(object):
                 elif op_id == Id.Arith_SlashEqual:
                     if mops.Equal(rhs_big, mops.ZERO):
                         e_die('Divide by zero')  # TODO: location
-                    new_big = num.IntDivide(old_big, rhs_big)
+                    new_big = mops.Div(old_big, rhs_big)
 
                 elif op_id == Id.Arith_PercentEqual:
                     if mops.Equal(rhs_big, mops.ZERO):
                         e_die('Divide by zero')  # TODO: location
-                    new_big = num.IntRemainder(old_big, rhs_big)
+                    new_big = mops.Rem(old_big, rhs_big)
 
                 elif op_id == Id.Arith_DGreatEqual:
                     new_big = mops.RShift(old_big, rhs_big)
@@ -694,20 +700,43 @@ class ArithEvaluator(object):
                     with tagswitch(left) as case:
                         if case(value_e.BashArray):
                             array_val = cast(value.BashArray, UP_left)
-                            index = mops.BigTruncate(
+                            small_i = mops.BigTruncate(
                                 self.EvalToBigInt(node.right))
-                            s = word_eval.GetArrayItem(array_val.strs, index)
+                            s = word_eval.GetArrayItem(array_val.strs, small_i)
 
                         elif case(value_e.BashAssoc):
                             left = cast(value.BashAssoc, UP_left)
                             key = self.EvalWordToString(node.right)
                             s = left.d.get(key)
 
+                        elif case(value_e.Str):
+                            left = cast(value.Str, UP_left)
+                            if self.exec_opts.strict_arith():
+                                e_die(
+                                    "Value of type Str can't be indexed (strict_arith)",
+                                    node.op)
+                            index = self.EvalToBigInt(node.right)
+                            # s[0] evaluates to s
+                            # s[1] evaluates to Undef
+                            s = left.s if mops.Equal(index,
+                                                     mops.ZERO) else None
+
+                        elif case(value_e.Undef):
+                            if self.exec_opts.strict_arith():
+                                e_die(
+                                    "Value of type Undef can't be indexed (strict_arith)",
+                                    node.op)
+                            s = None  # value.Undef
+
+                            # There isn't a way to distinguish Undef vs. empty
+                            # string, even with set -o nounset?
+                            # s = ''
+
                         else:
                             # TODO: Add error context
                             e_die(
-                                'Expected array or assoc in index expression, got %s'
-                                % ui.ValType(left))
+                                "Value of type %s can't be indexed" %
+                                ui.ValType(left), node.op)
 
                     if s is None:
                         val = value.Undef
@@ -734,12 +763,12 @@ class ArithEvaluator(object):
                 elif op_id == Id.Arith_Slash:
                     if mops.Equal(rhs_big, mops.ZERO):
                         e_die('Divide by zero', node.op)
-                    result = num.IntDivide(lhs_big, rhs_big)
+                    result = mops.Div(lhs_big, rhs_big)
 
                 elif op_id == Id.Arith_Percent:
                     if mops.Equal(rhs_big, mops.ZERO):
                         e_die('Divide by zero', node.op)
-                    result = num.IntRemainder(lhs_big, rhs_big)
+                    result = mops.Rem(lhs_big, rhs_big)
 
                 elif op_id == Id.Arith_DStar:
                     if mops.Greater(mops.ZERO, rhs_big):
@@ -773,8 +802,14 @@ class ArithEvaluator(object):
 
                 # Note: how to define shift of negative numbers?
                 elif op_id == Id.Arith_DLess:
+                    if mops.Greater(mops.ZERO, rhs_big):  # rhs_big < 0
+                        raise error.Expr("Can't left shift by negative number",
+                                         node.op)
                     result = mops.LShift(lhs_big, rhs_big)
                 elif op_id == Id.Arith_DGreat:
+                    if mops.Greater(mops.ZERO, rhs_big):  # rhs_big < 0
+                        raise error.Expr(
+                            "Can't right shift by negative number", node.op)
                     result = mops.RShift(lhs_big, rhs_big)
                 else:
                     raise AssertionError(op_id)
@@ -924,7 +959,7 @@ class BoolEvaluator(ArithEvaluator):
             exec_opts,  # type: optview.Exec
             mutable_opts,  # type: Optional[state.MutableOpts]
             parse_ctx,  # type: Optional[parse_lib.ParseContext]
-            errfmt,  # type: ErrorFormatter
+            errfmt,  # type: ui.ErrorFormatter
             always_strict=False  # type: bool
     ):
         # type: (...) -> None
@@ -932,6 +967,63 @@ class BoolEvaluator(ArithEvaluator):
                                 errfmt)
         self.always_strict = always_strict
 
+    def _IsDefined(self, s, blame_loc):
+        # type: (str, loc_t) -> bool
+
+        m = util.RegexSearch(consts.TEST_V_RE, s)
+        if m is None:
+            if self.exec_opts.strict_word_eval():
+                e_die('-v expected name or name[index]', blame_loc)
+            return False
+
+        var_name = m[1]
+        index_str = m[3]
+
+        val = self.mem.GetValue(var_name)
+        if len(index_str) == 0:  # it's just a variable name
+            return val.tag() != value_e.Undef
+
+        UP_val = val
+        with tagswitch(val) as case:
+            if case(value_e.BashArray):
+                val = cast(value.BashArray, UP_val)
+
+                # TODO: use mops.BigStr
+                try:
+                    index = int(index_str)
+                except ValueError as e:
+                    if self.exec_opts.strict_word_eval():
+                        e_die(
+                            '-v got BashArray and invalid index %r' %
+                            index_str, blame_loc)
+                    return False
+
+                if index < 0:
+                    if self.exec_opts.strict_word_eval():
+                        e_die('-v got invalid negative index %s' % index_str,
+                              blame_loc)
+                    return False
+
+                if index < len(val.strs):
+                    return val.strs[index] is not None
+
+                # out of range
+                return False
+
+            elif case(value_e.BashAssoc):
+                val = cast(value.BashAssoc, UP_val)
+                return index_str in val.d
+
+            else:
+                # work around mycpp bug!  parses as 'elif'
+                pass
+
+                if self.exec_opts.strict_word_eval():
+                    raise error.TypeErr(val, 'Expected BashArray or BashAssoc',
+                                        blame_loc)
+                return False
+        raise AssertionError()
+
     def _StringToBigIntOrError(self, s, blame_word=None):
         # type: (str, Optional[word_t]) -> mops.BigInt
         """Used by both [[ $x -gt 3 ]] and (( $x ))."""
@@ -989,9 +1081,9 @@ class BoolEvaluator(ArithEvaluator):
                 op_id = node.op_id
                 s = self._EvalCompoundWord(node.child)
 
-                # Now dispatch on arg type
-                arg_type = consts.BoolArgType(
-                    op_id)  # could be static in the LST?
+                # Now dispatch on arg type.  (arg_type could be static in the
+                # LST?)
+                arg_type = consts.BoolArgType(op_id)
 
                 if arg_type == bool_arg_type_e.Path:
                     return bool_stat.DoUnaryOp(op_id, s)
@@ -1017,8 +1109,7 @@ class BoolEvaluator(ArithEvaluator):
                             return self.exec_opts.opt0_array[index]
 
                     if op_id == Id.BoolUnary_v:
-                        val = self.mem.GetValue(s)
-                        return val.tag() != value_e.Undef
+                        return self._IsDefined(s, loc.Word(node.child))
 
                     e_die("%s isn't implemented" %
                           ui.PrettyId(op_id))  # implicit location
@@ -1095,8 +1186,9 @@ class BoolEvaluator(ArithEvaluator):
                         try:
                             indices = libc.regex_search(s2, regex_flags, s1, 0)
                         except ValueError as e:
-                            # Status 2 indicates a regex parse error.  This is fatal in OSH but
-                            # not in bash, which treats [[ like a command with an exit code.
+                            # Status 2 indicates a regex parse error.  This is
+                            # fatal in OSH but not in bash, which treats [[
+                            # like a command with an exit code.
                             e_die_status(2, e.message, loc.Word(node.right))
 
                         if indices is not None:
