@@ -23,7 +23,7 @@ from mycpp.mylib import iteritems, print_stderr
 from typing import Dict, List, Optional, TYPE_CHECKING
 if TYPE_CHECKING:
     from _devbuild.gen.syntax_asdl import command_t
-    from core.ui import ErrorFormatter
+    from display import ui
     from frontend.parse_lib import ParseContext
 
 _ = log
@@ -47,13 +47,17 @@ class TrapState(object):
         self.hooks = {}  # type: Dict[str, command_t]
         self.traps = {}  # type: Dict[int, command_t]
 
-    def ClearForSubProgram(self):
-        # type: () -> None
+    def ClearForSubProgram(self, inherit_errtrace):
+        # type: (bool) -> None
         """SubProgramThunk uses this because traps aren't inherited."""
 
-        # bash clears DEBUG hook in subshell, command sub, etc.  See
-        # spec/builtin-trap-bash.
+        # bash clears hooks like DEBUG in subshells.
+        # The ERR can be preserved if set -o errtrace
+        hook_err = self.hooks.get('ERR')
         self.hooks.clear()
+        if hook_err is not None and inherit_errtrace:
+            self.hooks['ERR'] = hook_err
+
         self.traps.clear()
 
     def GetHook(self, hook_name):
@@ -71,13 +75,17 @@ class TrapState(object):
 
     def AddUserTrap(self, sig_num, handler):
         # type: (int, command_t) -> None
-        """E.g.
-
-        SIGUSR1.
-        """
+        """ e.g. SIGUSR1 """
         self.traps[sig_num] = handler
 
-        if sig_num == SIGWINCH:
+        if sig_num == SIGINT:
+            # Don't disturb the underlying runtime's SIGINT handllers
+            # 1. CPython has one for KeyboardInterrupt
+            # 2. mycpp runtime simulates KeyboardInterrupt:
+            #    pyos::InitSignalSafe() calls RegisterSignalInterest(SIGINT),
+            #    then we PollSigInt() in the osh/cmd_eval.py main loop
+            self.signal_safe.SetSigIntTrapped(True)
+        elif sig_num == SIGWINCH:
             self.signal_safe.SetSigWinchCode(SIGWINCH)
         else:
             pyos.RegisterSignalInterest(sig_num)
@@ -88,20 +96,31 @@ class TrapState(object):
         mylib.dict_erase(self.traps, sig_num)
 
         if sig_num == SIGINT:
-            # Don't disturb the runtime signal handlers:
-            # 1. from CPython
-            # 2. pyos::InitSignalSafe() calls RegisterSignalInterest(SIGINT)
+            self.signal_safe.SetSigIntTrapped(False)
             pass
         elif sig_num == SIGWINCH:
             self.signal_safe.SetSigWinchCode(pyos.UNTRAPPED_SIGWINCH)
         else:
-            pyos.Sigaction(sig_num, SIG_DFL)
+            # TODO: In process.InitInteractiveShell(), 4 signals are set to
+            # SIG_IGN, not SIG_DFL:
+            #
+            # SIGQUIT SIGTSTP SIGTTOU SIGTTIN
+            #
+            # Should we restore them?  It's rare that you type 'trap' in
+            # interactive shells, but it might be more correct.  See what other
+            # shells do.
+            pyos.sigaction(sig_num, SIG_DFL)
 
     def GetPendingTraps(self):
         # type: () -> Optional[List[command_t]]
-        """Transfer ownership of the current queue of pending trap handlers to
-        the caller."""
+        """Transfer ownership of queue of pending trap handlers to caller."""
         signals = self.signal_safe.TakePendingSignals()
+        if 0:
+            log('*** GetPendingTraps')
+            for si in signals:
+                log('SIGNAL %d', si)
+            #import traceback
+            #traceback.print_stack()
 
         # Optimization for the common case: do not allocate a list.  This function
         # is called in the interpreter loop.
@@ -121,6 +140,17 @@ class TrapState(object):
 
         return run_list
 
+    def ThisProcessHasTraps(self):
+        # type: () -> bool
+        """
+        noforklast optimizations are not enabled when the process has code to
+        run after fork!
+        """
+        if 0:
+            log('traps %d', len(self.traps))
+            log('hooks %d', len(self.hooks))
+        return len(self.traps) != 0 or len(self.hooks) != 0
+
 
 def _GetSignalNumber(sig_spec):
     # type: (str) -> int
@@ -157,7 +187,7 @@ _HOOK_NAMES = ['EXIT', 'ERR', 'RETURN', 'DEBUG']
 class Trap(vm._Builtin):
 
     def __init__(self, trap_state, parse_ctx, tracer, errfmt):
-        # type: (TrapState, ParseContext, dev.Tracer, ErrorFormatter) -> None
+        # type: (TrapState, ParseContext, dev.Tracer, ui.ErrorFormatter) -> None
         self.trap_state = trap_state
         self.parse_ctx = parse_ctx
         self.arena = parse_ctx.arena
